# You might use, distribute and modify this code under the terms of
# the LICENSE.txt provided within the project monitoring-7s.
#
# If you do not have received a copy of the LICENSE.txt along with
# this file please write to info@sevensols.com and consider that
# this file can not be copied and/or distributed in any forms.
#
# Created by Eva Lopez Ruiz in July 2021.
#


"""Comunicate with InfluxDB API."""


import time
import json
from http.client import HTTPConnection
from wrz_mon.utils import LOGGER


"""Parameters to HTTP requests"""
METHODS = ['HEAD', 'POST', 'GET', 'PATCH']
HEAD, POST, GET, PATCH = METHODS


"""API URLs"""
SETUP = '/api/v2/setup'
READY = '/influxdb/v2.0/api/ready'
USERS = '/api/v2/users/'
ORGS = '/api/v2/orgs/'
BUCKETS = '/api/v2/buckets/'
TASK = '/api/v2/tasks'


class InfluxClient():
    """An object of InfluxClient class represent the communication with
    InfluxDB service.

    Atributes:
        url (str): URL of service
        name (str): Service name
    """

    def __init__(self, url, token=None, user_id=None, org_id=None,
                 bucket_id=None, auth_id=None):
        """Initialize object."""
        self.url = url
        self.name = 'InfluxDB'

        self.header = {'Accept': 'application/json',
                       'Content-Type': 'application/json'}
        self.token = token
        if token:
            self.header['Authorization'] = 'Token {}'.format(self.token)

        self.user_id = user_id
        self.org_id = org_id
        self.bucket_id = bucket_id
        self.auth_id = auth_id

    def set_initial_setup(self, username, password, influxdb_token,
                          organization, bucket):
        """Set initial configuration.

        Parameters:
            username (str)
            password (str)
            influxdb_token
            organization (str)
            bucket (str)

        Returns:
            Return a list of IDs on success (user ID, organization ID,
            bucket ID and auth ID), otherwise None
        """
        """retentionPeriodSeconds is 7 days in seconds"""
        msg = {'username': username, 'password': password,
               'token': influxdb_token, 'org': organization,
               'bucket': bucket, 'retentionPeriodSeconds': 604800}

        response, err = self._request(SETUP, POST, json.dumps(msg))
        if response is None:
            LOGGER.error('Failure to configure InfluxDB. Exception: %s' % err)
            return None

        self.token = influxdb_token
        self.header['Authorization'] = 'Token {}'.format(self.token)
        LOGGER.info('InfluxDB configured')

        return [response['user']['id'], response['org']['id'],
                response['bucket']['id'], response['auth']['id']]

    def update_user(self, username, password):
        """Update username and/or password

        Parameters:
            username (str)
            password (str)

        Returns:
            0 on success, otherwise -1
        """
        msg = {'oauthID': self.auth_id, 'name': username}
        url = USERS + self.user_id

        response, err = self._request(url, PATCH, json.dumps(msg))
        if response is None:
            LOGGER.error('Failure to update username. Exception: %s' % err)
            return -1

        msg = {'password': password}
        url = url + '/password'

        response, err = self._request(url, POST, json.dumps(msg))
        if response is None:
            LOGGER.error('Failure to update password. Exception: %s' % err)
            return -1

        LOGGER.info('InfluxDB user and password updated')
        return 0

    def update_organization(self, organization):
        """Update organization name.

        Parameters:
            organization (str): new organization name

        Returns:
            0 on success, otherwise -1
        """
        msg = {'name': organization}
        url = ORGS + self.org_id

        response, err = self._request(url, PATCH, json.dumps(msg))
        if response is None:
            LOGGER.error('Failure to update organization. Exception: %s' % err)
            return -1

        LOGGER.info('InfluxDB organization updated')
        return 0

    def update_bucket(self, bucket):
        """Update bucket name.

        Parameters:
            bucket (str): new bucket name

        Returns:
            0 on success, otherwise -1
        """
        msg = {'name': bucket}
        url = BUCKETS + self.bucket_id

        response, err = self._request(url, PATCH, json.dumps(msg))
        if response is None:
            LOGGER.error('Failure to update bucket. Exception: %s' % err)
            return -1

        LOGGER.info('InfluxDB bucket updated')
        return 0

    def create_task(self, task):
        """Import task

        Parameters:
            task (json): Task to import

        Returns:
            0 on success, otherwise -1
        """
        response, err = self._request(TASK, POST, json.dumps(task))
        if response is None:
            LOGGER.error("Failure to import {0}. Exception: {1}".format(task['description'], err))
            return -1

        LOGGER.info("%s imported succefully" % task['description'])
        return 0

    def check_service(self):
        """Check if service is running. It tries three times.

        Returns:
            retval (int): 0 on success, otherwise -1
        """
        sleep_intervals = [5, 10, 15]

        response, err = self._request()

        if err is not None:
            for i in sleep_intervals:
                LOGGER.warn("InfluxDB doesn't response. Sleep %s seconds" % i)
                time.sleep(i)
                response, err = self._request()
                if err is None:
                    break

        if response is None:
            LOGGER.critical('Failure to connect to InfluxDB service')
            return -1

        return 0

    def _request(self, api_url='/', method=HEAD, body=None):
        """Send HTTP request to server.

        Parameters:
            url (str): Server to send request
            api_url (str): ULR to request
            method (str): HTTP method
            body (str): request's argumets
            headers (json): Headers

        Returns:
            response (string): Server's respose
            reason (or err) (str): None value on success, otherwise error
            caught
        """
        response = None
        reason = None

        try:
            conn = HTTPConnection(self.url)
            conn.request(method, api_url, body, self.header)
            resp = conn.getresponse()

            if resp.status < 200 or 300 <= resp.status:
                reason = resp.reason
            else:
                response = resp.read().decode()
                if response:
                    response = json.loads(response)

            conn.close()

        except Exception as err:
            return None, err

        return response, reason
