#
# Monitoring Suite, Seven Solutions S.L.
#
# This file is part of monitoring-7s.
# You might use, distribute and modify this code under the terms of
# the LICENSE.txt provided within the project monitoring-7s.
#
# If you do not have received a copy of the LICENSE.txt along with
# this file please write to info@sevensols.com and consider that
# this file can not be copied and/or distributed in any forms.
#
# Created by Eva Lopez Ruiz in June 2021.
#

"""Telegraf Setup Manager."""


import os
import re
from shutil import copyfile

from wrz_mon.configuration import Configuration
from wrz_mon.configuration import TELEGRAF_CONTAINER, INFLUXDB_CONTAINER
from wrz_mon.configuration import ORGANIZATION_F, BUCKET_F, TOKEN_F
from wrz_mon.configuration import DEVICE_TYPE_F, TEMPLATE_F
from wrz_mon.utils import create_directory, delete_directory, TMP_DIR
from wrz_mon.utils import toml_load_file, toml_dump_file
from wrz_mon.utils import Container
from wrz_mon.utils import LOGGER


"""Configuration"""
CONFIG = Configuration()


"""Paths"""
TELEGRAF_DIR = 'telegraf'
TELEGRAF_D_DIR = 'telegraf.d'
CONF_FILE = 'telegraf.conf'
OUTPUT_CONF_FILE = 'outputs.influxdb.conf'
PROCESSORS_CONF_FILE = 'processors.conf'
TRAPS_CONF_FILE = 'inputs.traps.conf'
INSTALLATION_PATH = '/etc/telegraf'


def configure_telegraf():
    """Configure Telegraf and restart container."""
    tmp_dir = os.path.join(TELEGRAF_DIR, TMP_DIR)
    create_directory(tmp_dir)

    config_path = os.path.join(TELEGRAF_DIR, TELEGRAF_D_DIR)
    config_files = os.listdir(config_path)

    for f in config_files:
        src = os.path.join(TELEGRAF_DIR, TELEGRAF_D_DIR, f)
        dst = os.path.join(TELEGRAF_DIR, TMP_DIR, f)

        tag = re.fullmatch(r'inputs.poll.wrs.BCwri1.\S+.conf', f)
        if tag is not None:
            for dev in CONFIG.devices:
                if ('wrs' == CONFIG.devices[dev][DEVICE_TYPE_F] and
                    'BCwri1' == CONFIG.devices[dev][TEMPLATE_F]):
                    dst = os.path.join(TELEGRAF_DIR, TMP_DIR, dev+f)
                    create_input_poll_conf_file(dev, CONFIG.devices[dev], src, dst)
            continue

        tag = re.fullmatch(r'inputs.poll.wrs.GM.\S+.conf', f)
        if tag is not None:
            for dev in CONFIG.devices:
                if ('wrs' == CONFIG.devices[dev][DEVICE_TYPE_F] and
                    'GM' == CONFIG.devices[dev][TEMPLATE_F]):
                    dst = os.path.join(TELEGRAF_DIR, TMP_DIR, dev+f)
                    create_input_poll_conf_file(dev, CONFIG.devices[dev], src, dst)
            continue

        tag = re.fullmatch(r'inputs.poll.(\w+).\S+.conf', f)
        if tag is not None:
            for dev in CONFIG.devices:
                if tag[1] == CONFIG.devices[dev][DEVICE_TYPE_F]:
                    dst = os.path.join(TELEGRAF_DIR, TMP_DIR, dev+f)
                    create_input_poll_conf_file(dev, CONFIG.devices[dev], src, dst)
            continue

        if TRAPS_CONF_FILE == f:
            copyfile(src, dst)
            continue

        if PROCESSORS_CONF_FILE == f:
            copyfile(src, dst)
            continue

        if OUTPUT_CONF_FILE == f:
            if create_output_conf_file(src, dst) != 0:
                delete_directory(tmp_dir)
                return -1

    if load_configuration() != 0:
        delete_directory(tmp_dir)
        return -1

    delete_directory(tmp_dir)
    LOGGER.info('Telegraf succesfully configured')
    return 0


def create_input_poll_conf_file(name, c, src, dst):
    """Create Telegraf input configuration file (SNMP poll).

    Merge config.yml and telegraf.d files, and save them in ./telegraf/tmp
    directory.

    Parameters:
        name (str): the name of group
        c (dict): Configuration schema
        src (str): Source file
        dst (str): Destination file
    """
    config = toml_load_file(src)
    if (config is None):
        return

    if 'inputs' not in config.keys() or 'snmp' not in config['inputs'].keys():
        return

    for i in config['inputs']['snmp']:
        for j in i:

            if j == 'tags':
                if 'group' in c.keys():
                    i[j]['group'] = c['group']
                else:
                    i[j]['group'] = name
                continue

            if j not in c.keys():
                continue

            if j == 'agents':
                agents = []
                for k in c['agents']:
                    agent = '{0}://{1}:{2}'.format(c['agents'][k]['protocol'],
                                                   c['agents'][k]['ip'],
                                                   c['agents'][k]['port'])
                    agents.append(agent)
                i[j] = agents
            else:
                i[j] = c[j]

    retval, err = toml_dump_file(dst, config)
    if retval != 0:
        LOGGER.warn('Failure to write {0}. Exception: {1}'.format(dst, err))


def create_input_traps_conf_file(c, src, dst):
    """Create Telegraf input configuration file (SNMP traps).

    Merge config.yml and telegraf.d files, and save them in ./telegraf/tmp
    directory.

    Parameters:
        c (dict): Configuration schema
        src (str): Source file
        dst (str): Destination file
    """
    config = toml_load_file(src)
    if config is None:
        return

    for i in config['inputs']['snmp_trap'][0]:

        if i not in c.keys():
            continue

        config['inputs']['snmp_trap'][0][i] = str(c[i])

    retval, err = toml_dump_file(dst, config)
    if retval != 0:
        LOGGER.warn('Failure to write {0}. Exception: {1}'.format(dst, err))


def create_output_conf_file(src, dst):
    """Create Telegraf output configuration file.

    Merge config.yml and telegraf.d files, and save them in ./telegraf/tmp
    directory.

    Parameters:
        src (str): Source file
        dst (str): Destination file
    """
    oc = toml_load_file(src)
    if oc is None:
        LOGGER.error('Unable to read Telegraf configuration to Influx output')
        return -1

    influx_addr = 'http://influxdb:' + str(CONFIG.influxdb_port)
    oc['outputs']['influxdb_v2'][0]['urls'] = [influx_addr]
    oc['outputs']['influxdb_v2'][0][ORGANIZATION_F] = CONFIG.organization
    oc['outputs']['influxdb_v2'][0][BUCKET_F] = CONFIG.bucket

    if CONFIG.influxdb_token:
        oc['outputs']['influxdb_v2'][0][TOKEN_F] = CONFIG.influxdb_token
    else:
        CONFIG.read_token(INFLUXDB_CONTAINER)

        if CONFIG.influxdb_token is None:
            LOGGER.error('Unable to read InfluxDB token')
            return -1

        oc['outputs']['influxdb_v2'][0][TOKEN_F] = CONFIG.influxdb_token

    retval, err = toml_dump_file(dst, oc)
    if retval != 0:
        LOGGER.error('Unable to write Telegraf configuration to Influx output')
        return -1

    return 0


def load_configuration():
    """Set Telegraf configuration.

    Copy Telegraf configuration files (stored in ./telegraf/tmp) in Telegraf
    container, and restart it
    """
    container = Container(TELEGRAF_CONTAINER)

    src = os.path.join(TELEGRAF_DIR, CONF_FILE)
    dst = TELEGRAF_CONTAINER + ':' + os.path.join(INSTALLATION_PATH, CONF_FILE)
    retval, err = container.copy(src, dst)
    if retval != 0:
        LOGGER.error('Failure to copy configuration files. Stderr: {0}'.format(err))
        return -1

    tmp = os.path.join(TELEGRAF_DIR, TMP_DIR)
    for f in os.listdir(tmp):
        src = os.path.join(TELEGRAF_DIR, TMP_DIR, f)
        dst = TELEGRAF_CONTAINER + ':' + os.path.join(INSTALLATION_PATH, TELEGRAF_D_DIR)
        retval, err = container.copy(src, dst)
        if retval != 0:
            LOGGER.error('Failure to copy {0}. Stderr: {1}'.format(f, err))
            return -1

    LOGGER.info('Telegraf configuration files copied to docker instance')

    retval, err = container.restart()
    if retval != 0:
        LOGGER.error('Failure to restart Telegraf container. Stderr: {0}'.format(err))
        return -1

    LOGGER.info('Telegraf restarted')
    return 0
