#
# Monitoring Suite, Seven Solutions S.L.
#
# This file is part of monitoring-7s.
# You might use, distribute and modify this code under the terms of
# the LICENSE.txt provided within the project monitoring-7s.
#
# If you do not have received a copy of the LICENSE.txt along with
# this file please write to info@sevensols.com and consider that
# this file can not be copied and/or distributed in any forms.
#
# Created by Eva Lopez Ruiz in June 2021.
#

"""Functions related to filesystem."""


import yaml
import toml
import json
import os
from shutil import rmtree
from jsonschema import validate
from wrz_mon.utils import LOGGER


TMP_DIR = 'tmp'


def yml_load_file(file, schema_file=None):
    """Read and validate (if schema is passed as argument) a configuration file.

    Parameters:
        file (str): File to read
        schema (str): File which contains the schema

    Returns:
        config (dict): The configuration on success, otherwise None value
        err (str): None value on success, otherwise exception caught
    """
    config = None
    schema = None

    try:
        with open(file, 'r') as f:
            config = yaml.full_load(f)
    except Exception as err:
        return None, err

    if schema_file is None:
        return config, None

    try:
        with open(schema_file, 'r') as f:
            schema = yaml.full_load(f)
            validate(config, schema)
    except Exception as err:
        LOGGER.warn('Failure to load {0}. Exception: {1}'.format(file, err))
        return None, err

    return config, None


def yml_dump_file(file, config):
    """Write a configuration in a file.

    Parameters:
        file (str): Configuration file to write
        config (dict): Configuration to write

    Returns:
        retval (int): 0 on success, otherwise -1
        err (str): None value on success, otherwise exception error caught
    """
    try:
        with open(file, 'w') as f:
            yaml.dump(config, f)
    except Exception as err:
        LOGGER.warn('Failure to dump {0}. Exception: {1}'.format(file, err))
        return -1

    return 0


def toml_load_file(file):
    """Read a TOML file.

    Parameters:
        file (str): Configuration file to write

    Returns:
        content (dict): file content on success, otherwise None value
        err (str): None value on success, otherwise exception error caught
    """
    try:
        with open(file, 'r') as f:
            content = toml.load(f)
    except Exception as err:
        LOGGER.warn('Failure to load {0}. Exception: {1}'.format(file, err))
        return None

    return content


def toml_dump_file(file, content):
    """Write content in a TOML file.

    Parameters:
        file (str): Configuration file to write
        content (dict): Configuration to write

    Returns:
        retval (int): 0 on success, otherwise -1
        err (str): None value on success, otherwise exception error caught
    """
    try:
        with open(file, 'w') as f:
            toml.dump(content, f)
    except Exception as err:
        return -1, err

    return 0, None


def json_load_file(file):
    """Read a JSON file.

    Parameters:
        file (str): Configuration file to write

    Returns:
        content (dict): file content on success, otherwise None value
        err (str): None value on success, otherwise exception error caught
    """
    try:
        with open(file, 'r') as f:
            content = json.load(f)
    except Exception as err:
        return None, err

    return content, None


def json_dump_file(file, content):
    """Write content in a JSON file.

    Parameters:
        file (str): Configuration file to write
        content (dict): Configuration to write

    Returns:
        err (str): None value on success, otherwise exception error caught
    """
    try:
        with open(file, 'w') as f:
            json.dump(content, f)
    except Exception as err:
        return err

    return None


def create_directory(directory):
    """Create a temporary directory.

    Parameters:
        directory (str): The directory where create tmp

    Returns:
        retval (int): 0 on success, otherwise -1
        err (str): None value on success, otherwise exception error caught
    """
    if os.path.isdir(directory) is True:
        return None

    try:
        os.mkdir(directory)
    except Exception as err:
        return err

    return None


def delete_directory(directory):
    """Delete a temporary directory.

    Parameters:
        directory (str): The directory which contains tmp

    Returns:
        retval (int): 0 on success, otherwise -1
        err (str): None value on success, otherwise exception error caught
    """
    try:
        rmtree(directory)
    except Exception as err:
        return err

    return None
