#!/usr/bin/env python3
#
# Monitoring Suite, Seven Solutions S.L.
#
# This file is part of monitoring-7s.
# You might use, distribute and modify this code under the terms of
# the LICENSE.txt provided within the project monitoring-7s.
#
# If you do not have received a copy of the LICENSE.txt along with
# this file please write to info@sevensols.com and consider that
# this file can not be copied and/or distributed in any forms.
#
# Created by Eva Lopez Ruiz in June 2021.
#

"""wrz-monitoring tool."""


import argparse
import sys
from wrz_mon.utils import Compose, set_logging_config, LOGGER
from wrz_mon.configuration import Configuration
from wrz_mon.configuration import DEFAULT_CONF_FILE, DEFAULT_SCHEMA_FILE
from wrz_mon import configure_telegraf
from wrz_mon import influxdb_set_initial_setup, influxdb_update
from wrz_mon import grafana_set_initial_setup, grafana_update
from wrz_mon import grafana_import_dashboards, grafana_update_datasource
from wrz_mon.autodiscovery import AutoDiscovery


"""Actions allowed"""
ACTIONS = ['up-services', 'update', 'test-configuration', 'configure-telegraf', 'auto-discovery', 'down-services']
UP_SERVICES, UPDATE, TEST_CONFIGURATION, CONFIGURE_TELEGRAF, AUTO_DISCOVERY, UNINSTALL = ACTIONS


if __name__ == "__main__":

    parser = argparse.ArgumentParser(prog='wrz_monitoring',
                                     usage='%(prog)s action [options]',
                                     description='Monitoring suite manager')

    parser.add_argument('action', type=str, help='Action to execute', choices=ACTIONS)
    parser.add_argument('-u', '--username', type=str, help='InfluxDB and Grafana user')
    parser.add_argument('-p', '--password', type=str, help='InfluxDB and Grafana password')
    parser.add_argument('--second-user', type=str, help='Grafana second user')
    parser.add_argument('--second-user-password', type=str, help='Grafana second user password')
    parser.add_argument('--new-username', type=str, help='New InfluxDB and Grafana user')
    parser.add_argument('--new-password', type=str, help='New InfluxDB and Grafana password')
    parser.add_argument('--use-auto-discovery', help='Run SNMP scan before installation', action='store_true')
    parser.add_argument('-c', '--config-file', type=str, help='Configuration file')
    #parser.add_argument('-w', '--wrz-monitoring-path', type=str, help='monitoring-7S repository path')
    parser.add_argument('-l', '--log-file', help='File to prints logs', action='store_true')

    args = parser.parse_args()

    """Set logging"""
    set_logging_config(args.log_file)

    """Set configuration. If it fails, stop program execution"""
    conf_file = DEFAULT_CONF_FILE
    schema_file = DEFAULT_SCHEMA_FILE

    if args.config_file:
        conf_file = args.config_file

    config = Configuration()
    retval = config.load(conf_file, schema_file)

    if retval is False:
        sys.exit(-1)

    LOGGER.info('Configuration OK')

    if args.username:
        config.username = args.username
    if args.password:
        config.password = args.password

    if args.second_user:
        config.second_user = args.second_user
    if args.second_user_password:
        config.second_password = args.second_user_password

    """Execute acction"""

    if args.action == UP_SERVICES:

        answer = input("It will deploy the Seven Solution monitoring suite. \nDo you want to continue? [Y/N] ")
        if answer.lower() != 'y' and answer.lower() != 'yes':
            sys.exit(0)

        if config.username is None or config.password is None:
            LOGGER.critical('User and password are required')
            sys.exit(-1)

        if args.use_auto_discovery:
            autodiscovery = AutoDiscovery()
            autodiscovery.run()

        compose = Compose()
        result, err = compose.up_containers()
        if result != 0:
            LOGGER.critical('Unable to up containers. Exception: {0}'.format(err))
            sys.exit(-1)

        LOGGER.info('Docker containers started')

        if influxdb_set_initial_setup() != 0:
            sys.exit(-1)
        if configure_telegraf() != 0:
            sys.exit(-1)
        if grafana_set_initial_setup() != 0:
            sys.exit(-1)

        print("\nwrz_monitoring: To visualize data in Grafana UI, insert InfluxDB token in the Grafana datasource.")
        print("InfluxDB Token: %s" % config.influxdb_token)

        config.save()

    elif args.action == CONFIGURE_TELEGRAF:

        answer = input("It may add/remove some monitored devices. \nDo you want to continue? [Y/N] ")
        if answer.lower() != 'y' and answer.lower() != 'yes':
            sys.exit(0)

        configure_telegraf()
        config.save()

    elif args.action == UPDATE:

        answer = input("It may modify the configuration and dashboards. \nDo you want to continue? [Y/N] ")
        if answer.lower() != 'y' and answer.lower() != 'yes':
            sys.exit(0)

        if config.username is None or config.password is None:
            LOGGER.critical('User and password are required')
            sys.exit(-1)

        if config.devices_equals_old() is False:
            configure_telegraf()

        if (None not in (args.new_username, args.new_password) or
                config.global_equals_old() is False):
            grafana_update(args.new_username, args.new_password)
            influxdb_update(args.new_username, args.new_password)

        grafana_import_dashboards(create_dashboards=False)
        grafana_update_datasource()

        config.save()

    elif args.action == AUTO_DISCOVERY:

        answer = input("It will overwrite the configuration file. \nDo you want to continue? [Y/N] ")
        if answer.lower() != 'y' and answer.lower() != 'yes':
            sys.exit(0)

        autodiscovery = AutoDiscovery()
        autodiscovery.run()

    elif args.action == UNINSTALL:

        answer = input("It will delete the Docker containers and their information. \nDo you want to continue? [Y/N] ")
        if answer.lower() != 'y' and answer.lower() != 'yes':
            sys.exit(0)

        compose = Compose()
        result, err = compose.down_containers()
        if result != 0:
            LOGGER.critical('Unable to down containers. Exception: {0}'.format(err))
            sys.exit(-1)
        LOGGER.info('Docker containers deleted')
